@@ -36,10 +36,17 @@ uint32_t	allreports_group;	     /* IGMPv3 member reports       */
 uint32_t	dvmrp_group;		     /* DVMRP grp addr in net order */
 uint32_t	dvmrp_genid;		     /* IGMP generation id          */
 
+/*
+ * Local variables.
+ */
+#ifdef REGISTER_HANDLER
+static int	sock_id = -1;
+#endif
+
 /*
  * Local function definitions.
  */
-static void	igmp_read(int sd);
+static void	igmp_read(int sd, void *arg);
 static int	igmp_log_level(uint32_t type, uint32_t code);
 
 /*
@@ -105,7 +112,8 @@ void igmp_init(void)
     router_timeout            = IGMP_OTHER_QUERIER_PRESENT_INTERVAL;
 
 #ifdef REGISTER_HANDLER
-    if (register_input_handler(igmp_socket, igmp_read) < 0)
+    sock_id = pev_sock_add(igmp_socket, igmp_read, NULL);
+    if (sock_id < 0)
 	logit(LOG_ERR, 0, "Failed registering IGMP handler");
 #endif
 }
@@ -113,7 +121,8 @@ void igmp_init(void)
 void igmp_exit(void)
 {
 #ifdef REGISTER_HANDLER
-    deregister_input_handler(igmp_socket);
+    if (sock_id >= 0)
+	pev_sock_del(sock_id);
 #endif
     close(igmp_socket);
     free(recv_buf);
@@ -215,7 +224,7 @@ int igmp_debug_kind(uint32_t type, uint32_t code)
 /*
  * Read an IGMP message from igmp_socket
  */
-static void igmp_read(int sd)
+static void igmp_read(int sd, void *arg)
 {
     struct cmsghdr *cmsg;
     struct msghdr msgh;
@@ -243,7 +252,7 @@ static void igmp_read(int sd)
 
     for (cmsg = CMSG_FIRSTHDR(&msgh); cmsg; cmsg = CMSG_NXTHDR(&msgh, cmsg)) {
 #ifdef IP_PKTINFO
-	struct in_pktinfo *ipi = (struct in_pktinfo *)CMSG_DATA(cmsg);
+	const struct in_pktinfo *ipi = (struct in_pktinfo *)CMSG_DATA(cmsg);
 	char tmp[IF_NAMESIZE + 1] = { 0 };
 
 	if (cmsg->cmsg_level != SOL_IP || cmsg->cmsg_type != IP_PKTINFO)
@@ -643,11 +652,16 @@ void send_igmp(uint32_t src, uint32_t dst, int type, int code, uint32_t group, i
 
     rc = sendto(igmp_socket, send_buf, len, 0, (struct sockaddr *)&sin, sizeof(sin));
     if (rc < 0) {
-	if (errno == ENETDOWN)
+	switch (errno) {
+	case ENETUNREACH:
+	case ENETDOWN:
 	    check_vif_state();
-	else
+	    break;
+	default:
 	    logit(igmp_log_level(type, code), errno, "sendto to %s on %s",
 		  inet_fmt(dst, s1, sizeof(s1)), inet_fmt(src, s2, sizeof(s2)));
+	    break;
+	}
     }
 
     if (setloop)
