@@ -54,7 +54,7 @@ static void discard_route            (struct rtentry *rt);
 static int  compare_rts              (const void *rt1, const void *rt2);
 static struct rtentry *report_chunk  (int, struct rtentry *, vifi_t, uint32_t, int *);
 static void queue_blaster_report     (vifi_t vifi, uint32_t src, uint32_t dst, char *p, size_t datalen, uint32_t level);
-static void process_blaster_report   (void *vifip);
+static void process_blaster_report   (int id, void *vifip);
 
 
 /*
@@ -400,8 +400,10 @@ void update_route(uint32_t origin, uint32_t mask, uint32_t metric, uint32_t src,
 	}
 
 	IF_DEBUG(DEBUG_RTDETAIL) {
-	    logit(LOG_DEBUG, 0, "%s advertises new route %s",
-		  inet_fmt(src, s1, sizeof(s1)), inet_fmts(origin, mask, s2, sizeof(s2)));
+	    logit(LOG_DEBUG, 0, "%s (origin %s) advertising new route %s",
+		  src ? inet_fmt(src, s1, sizeof(s1)) : "we are",
+		  inet_fmt(origin, s2, sizeof(s2)),
+		  inet_fmts(origin, mask, s2, sizeof(s2)));
 	}
 
 	/*
@@ -678,7 +680,7 @@ void age_routes(void)
 		r->rt_flags   |= RTF_CHANGED;
 		routes_changed = TRUE;
 	    }
-	} else if (virtual_time % (ROUTE_REPORT_INTERVAL * 2) == 0) {
+	} else if (virtual_time > 0 && (virtual_time % (ROUTE_REPORT_INTERVAL * 2) == 0)) {
 	    /*
 	     * Time out subordinateness that hasn't been reported in
 	     * the last 2 intervals.
@@ -818,7 +820,7 @@ static void queue_blaster_report(vifi_t vifi, uint32_t src, uint32_t dst, char *
 	}
 
 	*i = vifi;
-	uv->uv_blastertimer = timer_set(5, process_blaster_report, i);
+	uv->uv_blastertimer = pev_timer_add(5000000, 0, process_blaster_report, i);
     }
 }
 
@@ -827,7 +829,7 @@ static void queue_blaster_report(vifi_t vifi, uint32_t src, uint32_t dst, char *
  * queue.  If there are more routes remaining, reschedule myself to run
  * in 1 second.
  */
-static void process_blaster_report(void *vifip)
+static void process_blaster_report(int id, void *vifip)
 {
     vifi_t vifi = *(int *)vifip;
     struct blaster_hdr *bh;
@@ -859,11 +861,14 @@ static void process_blaster_report(void *vifip)
 	IF_DEBUG(DEBUG_ROUTE) {
 	    logit(LOG_DEBUG, 0, "Finish processing vif %d blaster", vifi);
 	}
+
+	pev_timer_del(id);
     } else {
 	IF_DEBUG(DEBUG_ROUTE) {
 	    logit(LOG_DEBUG, 0, "More blasted routes to come on vif %d", vifi);
 	}
-	uv->uv_blastertimer = timer_set(1, process_blaster_report, vifip);
+
+	pev_timer_set(id, 1000000);
     }
 }
 
