@@ -47,13 +47,13 @@ static void send_probe_on_vif  (struct uvif *v);
 static void send_query         (struct uvif *v, uint32_t dst, int code, uint32_t group);
 static int  info_version       (uint8_t *p, size_t plen);
 
-static void delete_group_cb    (void *arg);
+static void delete_group_cb    (int id, void *arg);
 static int  delete_group_timer (vifi_t vifi, struct listaddr *g);
 
-static void send_query_cb      (void *arg);
+static void send_query_cb      (int id, void *arg);
 static int  send_query_timer   (vifi_t vifi, struct listaddr *g, int delay, int num);
 
-static void group_version_cb   (void *arg);
+static void group_version_cb   (int id, void *arg);
 static int  group_version_timer(vifi_t vifi, struct listaddr *g);
 
 /*
@@ -135,11 +135,11 @@ void init_vifs(void)
 	}
 
 	if (uv->uv_flags & VIFF_TUNNEL)
-	    logit(LOG_INFO, 0, "vif #%d, tunnel %s -> %s", vifi,
+	    logit(LOG_INFO, 0, "%s: vif #%d, tunnel %s -> %s", uv->uv_name, vifi,
 		  inet_fmt(uv->uv_lcl_addr, s1, sizeof(s1)),
 		  inet_fmt(uv->uv_rmt_addr, s2, sizeof(s2)));
 	else
-	    logit(LOG_INFO, 0, "vif #%d, phyint %s", vifi,
+	    logit(LOG_INFO, 0, "%s: vif #%d, phyint %s", uv->uv_name, vifi,
 		  inet_fmt(uv->uv_lcl_addr, s1, sizeof(s1)));
 
 	start_vif2(vifi);
@@ -150,15 +150,15 @@ void init_vifs(void)
      * which this router is the elected querier.
      */
     if (query_timerid > 0)
-	timer_clear(query_timerid);
-    query_timerid = timer_set(igmp_query_interval, query_groups, NULL);
+	pev_timer_del(query_timerid);
+    query_timerid = pev_timer_add(1, igmp_query_interval * 1000000, query_groups, NULL);
 
     /*
      * Periodically probe all VIFs for DVMRP neighbors
      */
     if (dvmrp_timerid > 0)
-	timer_clear(dvmrp_timerid);
-    dvmrp_timerid = timer_set(NEIGHBOR_PROBE_INTERVAL, query_dvmrp, NULL);
+	pev_timer_del(dvmrp_timerid);
+    dvmrp_timerid = pev_timer_add(1, NEIGHBOR_PROBE_INTERVAL * 1000000, query_dvmrp, NULL);
 }
 
 /*
@@ -227,7 +227,7 @@ void blaster_free(struct uvif *uv)
     uv->uv_blasterbuf = NULL;
 
     if (uv->uv_blastertimer)
-	uv->uv_blastertimer = timer_clear(uv->uv_blastertimer);
+	uv->uv_blastertimer = pev_timer_del(uv->uv_blastertimer);
 }
 
 /*
@@ -254,14 +254,14 @@ void init_installvifs(void)
 	}
 
 	if (uv->uv_flags & VIFF_TUNNEL) {
-	    logit(LOG_INFO, 0, "vif #%d, tunnel %s -> %s", vifi,
+	    logit(LOG_INFO, 0, "%s: vif #%d, tunnel %s -> %s", uv->uv_name, vifi,
 		  inet_fmt(uv->uv_lcl_addr, s1, sizeof(s1)),
 		  inet_fmt(uv->uv_rmt_addr, s2, sizeof(s2)));
 
 	    /* Set tunnel vif name, Linux use dvmrpN */
 	    snprintf(uv->uv_name, sizeof(uv->uv_name), "dvmrp%d", vifi);
 	} else {
-	    logit(LOG_INFO, 0, "vif #%d, phyint %s", vifi,
+	    logit(LOG_INFO, 0, "%s: vif #%d, phyint %s", uv->uv_name, vifi,
 		  inet_fmt(uv->uv_lcl_addr, s1, sizeof(s1)));
 	}
 	k_add_vif(vifi, uv);
@@ -542,8 +542,8 @@ static void start_vif2(vifi_t vifi)
  */
 static void stop_vif(vifi_t vifi)
 {
-    struct listaddr *a, *tmp;
-    struct phaddr *p;
+    struct listaddr *al, *tmp;
+    struct phaddr *pa;
     struct uvif *uv;
 
     uv = find_uvif(vifi);
@@ -573,22 +573,37 @@ static void stop_vif(vifi_t vifi)
 	 */
 	start_route_updates();
 	update_route(uv->uv_subnet, uv->uv_subnetmask, UNREACHABLE, 0, vifi, NULL);
-	for (p = uv->uv_addrs; p; p = p->pa_next) {
+	for (pa = uv->uv_addrs; pa; pa = pa->pa_next) {
 	    start_route_updates();
-	    update_route(p->pa_subnet, p->pa_subnetmask, UNREACHABLE, 0, vifi, NULL);
+	    update_route(pa->pa_subnet, pa->pa_subnetmask, UNREACHABLE, 0, vifi, NULL);
 	}
 
 	/*
 	 * Discard all group addresses.  (No need to tell kernel;
 	 * the k_del_vif() call, below, will clean up kernel state.)
 	 */
-	TAILQ_FOREACH_SAFE(a, &uv->uv_groups, al_link, tmp) {
-	    TAILQ_REMOVE(&uv->uv_groups, a, al_link);
-	    free(a);
+	TAILQ_FOREACH_SAFE(al, &uv->uv_groups, al_link, tmp) {
+	    uint32_t group = al->al_addr;
+
+	    TAILQ_REMOVE(&uv->uv_groups, al, al_link);
+
+	    logit(LOG_INFO, 0, "Discarding group %s on %s (stopping)",
+		  inet_fmt(group, s1, sizeof(s1)), uv->uv_name);
+
+	    if (al->al_query > 0)
+		al->al_query = pev_timer_del(al->al_query);
+
+	    if (al->al_timerid > 0)
+		al->al_timerid = pev_timer_del(al->al_timerid);
+
+	    if (al->al_pv_timerid > 0)
+		al->al_pv_timerid = pev_timer_del(al->al_pv_timerid);
+
+	    free(al);
 	}
 
 	IF_DEBUG(DEBUG_IGMP) {
-	    logit(LOG_DEBUG, 0, "Releasing querier duties on vif %u", vifi);
+	    logit(LOG_DEBUG, 0, "Releasing querier duties on %s (vif %u)", uv->uv_name, vifi);
 	}
 	uv->uv_flags &= ~VIFF_QUERIER;
     }
@@ -609,10 +624,10 @@ static void stop_vif(vifi_t vifi)
     if (!NBRM_ISEMPTY(uv->uv_nbrmap))
 	neighbor_vifs--;
 
-    TAILQ_FOREACH_SAFE(a, &uv->uv_neighbors, al_link, tmp) {
-	TAILQ_REMOVE(&uv->uv_neighbors, a, al_link);
-	nbrs[a->al_index] = NULL;
-	free(a);
+    TAILQ_FOREACH_SAFE(al, &uv->uv_neighbors, al_link, tmp) {
+	TAILQ_REMOVE(&uv->uv_neighbors, al, al_link);
+	nbrs[al->al_index] = NULL;
+	free(al);
     }
     NBRM_CLRALL(uv->uv_nbrmap);
 }
@@ -623,9 +638,9 @@ static void stop_vif(vifi_t vifi)
  */
 void stop_all_vifs(void)
 {
-    struct listaddr *a, *tmp;
+    struct listaddr *al, *tmp;
     struct vif_acl *acl;
-    struct phaddr *ph;
+    struct phaddr *pa;
     struct uvif *uv;
     vifi_t vifi;
 
@@ -636,25 +651,35 @@ void stop_all_vifs(void)
 	}
 	uv->uv_querier = NULL;
 
-	TAILQ_FOREACH_SAFE(a, &uv->uv_join, al_link, tmp) {
-	    uint32_t group = a->al_addr;
+	TAILQ_FOREACH_SAFE(al, &uv->uv_join, al_link, tmp) {
+	    uint32_t group = al->al_addr;
 
 	    logit(LOG_INFO, 0, "Leaving static group %s on %s from %s",
 		  inet_fmt(group, s1, sizeof(s1)), uv->uv_name, config_file);
 	    k_leave(group, uv->uv_lcl_addr);
-	    TAILQ_REMOVE(&uv->uv_join, a, al_link);
-	    free(a);
+	    TAILQ_REMOVE(&uv->uv_join, al, al_link);
+	    free(al);
 	}
 
-	TAILQ_FOREACH_SAFE(a, &uv->uv_groups, al_link, tmp) {
-	    TAILQ_REMOVE(&uv->uv_groups, a, al_link);
-	    free(a);
+	TAILQ_FOREACH_SAFE(al, &uv->uv_groups, al_link, tmp) {
+	    TAILQ_REMOVE(&uv->uv_groups, al, al_link);
+
+	    if (al->al_query > 0)
+		al->al_query = pev_timer_del(al->al_query);
+
+	    if (al->al_timerid > 0)
+		al->al_timerid = pev_timer_del(al->al_timerid);
+
+	    if (al->al_pv_timerid > 0)
+		al->al_pv_timerid = pev_timer_del(al->al_pv_timerid);
+
+	    free(al);
 	}
 
-	TAILQ_FOREACH_SAFE(a, &uv->uv_neighbors, al_link, tmp) {
-	    TAILQ_REMOVE(&uv->uv_neighbors, a, al_link);
-	    nbrs[a->al_index] = NULL;
-	    free(a);
+	TAILQ_FOREACH_SAFE(al, &uv->uv_neighbors, al_link, tmp) {
+	    TAILQ_REMOVE(&uv->uv_neighbors, al, al_link);
+	    nbrs[al->al_index] = NULL;
+	    free(al);
 	}
 
 	while (uv->uv_acl) {
@@ -665,9 +690,9 @@ void stop_all_vifs(void)
 	uv->uv_acl = NULL;
 
 	while (uv->uv_addrs) {
-	    ph = uv->uv_addrs;
-	    uv->uv_addrs = ph->pa_next;
-	    free(ph);
+	    pa = uv->uv_addrs;
+	    uv->uv_addrs = pa->pa_next;
+	    free(pa);
 	}
 	uv->uv_addrs = NULL;
 
@@ -751,13 +776,11 @@ vifi_t find_vif_direct(uint32_t src, uint32_t dst)
  * so can not cause loss of membership (but can send more packets than
  * necessary)
  */
-void query_groups(void *arg)
+void query_groups(int id, void *arg)
 {
     struct uvif *uv;
     vifi_t vifi;
 
-    timer_set(igmp_query_interval, query_groups, arg);
-
     UVIF_FOREACH(vifi, uv) {
 	if (uv->uv_flags & (VIFF_DOWN | VIFF_DISABLED))
 	    continue;
@@ -773,10 +796,8 @@ void query_groups(void *arg)
  * come up.  (If they have, they will also be probed as part of
  * their initialization.)
  */
-void query_dvmrp(void *arg)
+void query_dvmrp(int id, void *arg)
 {
-    timer_set(NEIGHBOR_PROBE_INTERVAL, query_dvmrp, arg);
-
     probe_for_neighbors();
 
     if (vifs_down)
@@ -888,10 +909,10 @@ void accept_membership_query(int ifi, uint32_t src, uint32_t dst, uint32_t group
 	TAILQ_FOREACH(g, &uv->uv_groups, al_link) {
 	    if (group == g->al_addr && g->al_query == 0) {
 		if (g->al_timerid > 0)
-		    g->al_timerid = timer_clear(g->al_timerid);
+		    g->al_timerid = pev_timer_del(g->al_timerid);
 
 		if (g->al_query > 0)
-		    g->al_query = timer_clear(g->al_query);
+		    g->al_query = pev_timer_del(g->al_query);
 
 		/* setup a timeout to remove the group membership */
 		g->al_timer = IGMP_LAST_MEMBER_QUERY_COUNT * tmo / IGMP_TIMER_SCALE;
@@ -909,9 +930,10 @@ void accept_membership_query(int ifi, uint32_t src, uint32_t dst, uint32_t group
 
 static void group_debug(struct listaddr *g, char *s, int is_change)
 {
-    IF_DEBUG(DEBUG_IGMP)
+    IF_DEBUG(DEBUG_IGMP) {
 	logit(LOG_DEBUG, 0, "%sIGMP v%d compatibility mode for group %s",
 	      is_change ? "Change to " : "", g->al_pv, s);
+    }
 }
 
 /*
@@ -987,10 +1009,10 @@ void accept_group_report(int ifi, uint32_t src, uint32_t dst, uint32_t group, in
 	    g->al_timer = IGMP_GROUP_MEMBERSHIP_INTERVAL;
 
 	    if (g->al_query > 0)
-		g->al_query = timer_clear(g->al_query);
+		g->al_query = pev_timer_del(g->al_query);
 
 	    if (g->al_timerid > 0)
-		g->al_timerid = timer_clear(g->al_timerid);
+		g->al_timerid = pev_timer_del(g->al_timerid);
 
 	    g->al_timerid = delete_group_timer(vifi, g);
 
@@ -1000,7 +1022,7 @@ void accept_group_report(int ifi, uint32_t src, uint32_t dst, uint32_t group, in
 	     */
 	    if (g->al_pv < 3 && old_report) {
 		if (g->al_pv_timerid)
-		    g->al_pv_timerid = timer_clear(g->al_pv_timerid);
+		    g->al_pv_timerid = pev_timer_del(g->al_pv_timerid);
 
 		g->al_pv_timerid = group_version_timer(vifi, g);
 	    }
@@ -1129,7 +1151,7 @@ void accept_leave_message(int ifi, uint32_t src, uint32_t dst, uint32_t group)
 
 	/** delete old timer set a timer for expiration **/
 	if (g->al_timerid > 0)
-	    g->al_timerid = timer_clear(g->al_timerid);
+	    g->al_timerid = pev_timer_del(g->al_timerid);
 
 	/** send a group specific query **/
 	g->al_query = send_query_timer(vifi, g, igmp_last_member_interval,
@@ -1789,7 +1811,6 @@ struct listaddr *update_neighbor(vifi_t vifi, uint32_t addr, int msgtype, char *
 	    send_tables = addr;
 	}
 
-
 	NBRM_SET(i, uv->uv_nbrmap);
 	add_neighbor_to_routes(vifi, i);
     } else {
@@ -1823,8 +1844,12 @@ struct listaddr *update_neighbor(vifi_t vifi, uint32_t addr, int msgtype, char *
 
 	if ((n->al_flags & NBRF_ONEWAY) && msgtype == DVMRP_PROBE) {
 	    if (in_router_list) {
-		if (NBRM_ISEMPTY(uv->uv_nbrmap))
+		if (NBRM_ISEMPTY(uv->uv_nbrmap)) {
+		    send_tables = uv->uv_dst_addr;
 		    neighbor_vifs++;
+		} else {
+		    send_tables = addr;
+		}
 		NBRM_SET(n->al_index, uv->uv_nbrmap);
 		add_neighbor_to_routes(vifi, n->al_index);
 		logit(LOG_NOTICE, 0, "Peering with %s on vif %u is no longer one-way",
@@ -1903,7 +1928,10 @@ struct listaddr *update_neighbor(vifi_t vifi, uint32_t addr, int msgtype, char *
 	if (!send_tables)
 	    send_tables = addr;
     }
+
     if (send_tables) {
+	logit(LOG_NOTICE, 0, "Sending DVMRP probe on %s and route report to %s",
+	      uv->uv_name, inet_fmt(send_tables, s1, sizeof(s1)));
 	send_probe_on_vif(uv);
 	report(ALL_ROUTES, vifi, send_tables);
     }
@@ -2173,7 +2201,7 @@ void dump_vifs(FILE *fp, int detail)
 		    label,
 		    inet_fmt(a->al_addr, s1, sizeof(s1)),
 		    inet_fmt(a->al_reporter, s2, sizeof(s2)),
-		    scaletime(timer_get(a->al_timerid)));
+		    scaletime(pev_timer_get(a->al_timerid) / 1000000));
 	    label = "";
 	}
 	label = "boundaries:";
@@ -2239,7 +2267,7 @@ void dump_vifs(FILE *fp, int detail)
 /*
  * Time out old version compatibility mode
  */
-static void group_version_cb(void *arg)
+static void group_version_cb(int id, void *arg)
 {
     cbk_t *cbk = (cbk_t *)arg;
     vifi_t vifi = cbk->vifi;
@@ -2256,7 +2284,7 @@ static void group_version_cb(void *arg)
 	  cbk->g->al_pv - 1, cbk->g->al_pv, inet_fmt(cbk->g->al_addr, s1, sizeof(s1)), uv->uv_name);
 
     if (cbk->g->al_pv < 3)
-	timer_set(IGMP_GROUP_MEMBERSHIP_INTERVAL, group_version_cb, cbk);
+	pev_timer_set(id, IGMP_GROUP_MEMBERSHIP_INTERVAL * 1000000);
     else
 	free(cbk);
 }
@@ -2277,13 +2305,13 @@ static int group_version_timer(vifi_t vifi, struct listaddr *g)
     cbk->vifi = vifi;
     cbk->g = g;
 
-    return timer_set(IGMP_GROUP_MEMBERSHIP_INTERVAL, group_version_cb, cbk);
+    return pev_timer_add(IGMP_GROUP_MEMBERSHIP_INTERVAL * 1000000, 0, group_version_cb, cbk);
 }
 
 /*
  * Time out record of a group membership on a vif
  */
-static void delete_group_cb(void *arg)
+static void delete_group_cb(int id, void *arg)
 {
     cbk_t *cbk = (cbk_t *)arg;
     struct listaddr *g = cbk->g;
@@ -2298,10 +2326,10 @@ static void delete_group_cb(void *arg)
 	  inet_fmt(cbk->g->al_addr, s1, sizeof(s1)), uv->uv_name);
 
     if (g->al_query > 0)
-	g->al_query = timer_clear(g->al_query);
+	g->al_query = pev_timer_del(g->al_query);
 
     if (g->al_pv_timerid > 0)
-	g->al_pv_timerid = timer_clear(g->al_pv_timerid);
+	g->al_pv_timerid = pev_timer_del(g->al_pv_timerid);
 
     delete_lclgrp(vifi, g->al_addr);
     TAILQ_REMOVE(&uv->uv_groups, g, al_link);
@@ -2315,8 +2343,16 @@ static void delete_group_cb(void *arg)
  */
 static int delete_group_timer(vifi_t vifi, struct listaddr *g)
 {
+    uint32_t group = g->al_addr;
+    struct uvif *uv;
     cbk_t *cbk;
 
+    uv = find_uvif(vifi);
+    if (!uv) {
+	logit(LOG_ERR, 0, "%s(): Failed, invalid VIFI %d", __func__, vifi);
+	return -1;
+    }
+
     cbk = calloc(1, sizeof(cbk_t));
     if (!cbk) {
 	logit(LOG_ERR, errno, "%s(): Failed allocating memory", __func__);
@@ -2329,33 +2365,33 @@ static int delete_group_timer(vifi_t vifi, struct listaddr *g)
     /* Record mtime for IPC "show igmp" */
     g->al_mtime = virtual_time;
 
-    return timer_set(g->al_timer, delete_group_cb, cbk);
+    logit(LOG_INFO, 0, "Setting timeout %d for group %s on %s (vif %d)",
+	  g->al_timer, inet_fmt(group, s1, sizeof(s1)), uv->uv_name, vifi);
+
+    return pev_timer_add(g->al_timer * 1000000, 0, delete_group_cb, cbk);
 }
 
 /*
  * Send a group-specific query.
  */
-static int do_send_gsq(cbk_t *cbk)
+static void send_query_cb(int id, void *arg)
 {
+    cbk_t *cbk = (cbk_t *)arg;
     struct uvif *uv;
 
     uv = find_uvif(cbk->vifi);
     if (!uv)
-	return -1;
+	goto done;
 
     send_query(uv, cbk->g->al_addr, cbk->delay * IGMP_TIMER_SCALE, cbk->g->al_addr);
-    if (--cbk->num == 0) {
-	cbk->g->al_query = 0;	/* we're done, clear us from group */
-	free(cbk);
-	return 0;
+    if (--cbk->num) {
+	pev_timer_set(id, cbk->delay * 1000000);
+	return;
     }
 
-    return timer_set(cbk->delay, send_query_cb, cbk);
-}
-
-static void send_query_cb(void *arg)
-{
-    do_send_gsq((cbk_t *)arg);
+  done:
+    cbk->g->al_query = pev_timer_del(cbk->g->al_query);
+    free(cbk);
 }
 
 /*
@@ -2376,7 +2412,7 @@ static int send_query_timer(vifi_t vifi, struct listaddr *g, int delay, int num)
     cbk->delay = delay;
     cbk->num   = num;
 
-    return do_send_gsq(cbk);
+    return pev_timer_add(1, 0, send_query_cb, cbk);
 }
 
 /**
